using Optim
################################
# min  x^2
# s.t. x = 1
#
# punto di KKT: x = 1, mu = -2
################################

@printf("\n\nMinimize x^2\n   s.t.  x = 1\n\n")

mu = 0.
rho = 0.1*1.1

P(x)  = x[1]^2               + rho*(x[1]-1)^2
La(x) = x[1]^2 + mu*(x[1]-1) + rho*(x[1]-1)^2
function grad_La!(x,g) 
	g[1] = 2*x[1] + mu + 2*rho*(x[1]-1)
	return nothing
end
function grad_P!(x,g) 
	g[1] = 2*x[1] + 2*rho*(x[1]-1)
	return nothing
end

xzero = [10.]
xk    = xzero
g     = [0.]
tol   = 1.0
j     = 1

@printf("it       mu           rho          tol            xk           La(xk)   \n")
@printf("--------------------------------------------------------------------------\n")
#         0  1.000000e+01  1.000000e-01 -1.960000e+01  4.000000e-01  4.000000e-01
for k = 1:200
	res = optimize(La,grad_La!,xk,g_tol = tol)
	xk  = res.minimum
	grad_La!(xk,g)
	@printf("%2d %13.6e %13.6e %13.6e %13.6e %13.6e %13.6e\n",k,mu,rho,tol,xk[1],La(xk),norm(g))
	#println(k," ",mu," ",rho," ",xk)
	if(norm(g) < 1.e-6) && (abs(xk[1]-1) < 1.e-6)
		break
	end
	if (abs(xk[1]-1) >= tol)
		rho = 0.1*1.1^j
		j   = j+1
	end
	mu  = mu + rho*(xk[1]-1)
	tol = max(tol/2.0,1.e-9)
end
println("xstar = ",xk)
println("h(xstar) = ",xk[1]-1)

print("\n\nHit return key to proceed to next example")
readline()

################################
# min  x
# s.t. x(x-1) = 0
#
# punto di KKT: x = 0, mu = -2
################################
@printf("\n\nMinimize x\n   s.t.  x(x-1) = 0\n\n")

mu   = 0.0
rho  = 0
rhop = 0

f(x) = x[1]
h(x) = x[1]*(x[1]-1.0)

La(x) = f(x) + mu*h(x) + rho*(h(x)^2)
P(x)  = f(x) + rhop*(h(x)^2)
function grad_P!(x,g)
	g[1] = 1.0 + 2.0*rhop*(h(x))*(x[1]-1.0 + x[1])
	return nothing
end

function grad_La!(x,g)
	g[1] = 1.0 + mu*(x[1]-1.0 + x[1]) + 2.0*rho*(h(x))*(x[1]-1.0 + x[1])
	return nothing
end
xzero = [10.]
xklagr = xzero
xkpen  = xzero
tol    = 1.0
g      = [0.]
Lconv  = false
Pconv  = false

@printf("it       mu           rho          La             P    \n")
@printf("------------------------------------------------------------\n")
#         0  1.000000e+01  1.000000e-01 -1.960000e+01  4.000000e-01
for k = 0:100
	rhoit = 0.1*1.6^k

	if Lconv && Pconv
		break
	end

	if ~Lconv && ~Pconv
		@printf("%2d %13.6e %13.6e %13.6e %13.6e\n",k,mu,rhoit,La(xklagr),P(xkpen))
	elseif ~Pconv
		@printf("%2d %13.6e %13.6e               %13.6e\n",k,mu,rhoit,P(xkpen))
	elseif ~Lconv	
		@printf("%2d %13.6e %13.6e %13.6e\n",k,mu,rhoit,La(xklagr))
	else
	end
	#," ",xklagr," ",xkpen)

	if ~Lconv	
		rho = rhoit
		xstart = xklagr
		res = optimize(La,grad_La!,xstart,g_tol = tol,show_trace = false)
		xklagr = res.minimum
		grad_La!(xklagr,g)
		mu  = mu + rho*h(xklagr)
		if(norm(g) < 1.e-7) && (abs(h(xklagr)) < 1.e-7)
			#Lagr method converged
			Lconv = true
		end
	end

	if ~Pconv
		rhop = rhoit
		xstart = xkpen
		res = optimize(P,grad_P!,xstart,g_tol = tol,show_trace = false)
		xkpen = res.minimum
		if(abs(h(xkpen)) < 1.e-7)
			#Pen method converged
			Pconv = true
		end
	end
	tol = tol/2.0

end

println("xlagr = ",xklagr," h(xlagr) = ",h(xklagr))
println(" xpen = ",xkpen ," h( xpen) = ",h(xkpen))

print("\n\nHit return key to proceed to next example")
readline()

################################
# min  x
# s.t. x(x-1)(x-2) = 0
#
# punto di KKT: x = 1, mu = -2
################################
@printf("\n\nMinimize x\n   s.t.  x(x-1)(x-2) = 0\n\n")

mu = 0.0
rho = 0

f(x) = x[1]
h(x) = x[1]*(x[1]-1.0)*(x[1]-2.0)

La(x) = f(x) + mu*h(x) + rho*(h(x)^2)
P(x)  = f(x) + rho*(h(x)^2)
function grad_P!(x,g)
	g[1] = 1.0 + 2.0*rho*(h(x))*((x[1]-1.0)*(x[1]-2.0) + x[1]*(x[1]-2.0) + x[1]*(x[1]-1.0))
	return nothing
end

function grad_La!(x,g)
	g[1] = begin
		   1.0 + mu*((x[1]-1.0)*(x[1]-2.0) + x[1]*(x[1]-2.0) + x[1]*(x[1]-1.0)) +
           2.0*rho*(h(x))*((x[1]-1.0)*(x[1]-2.0) + x[1]*(x[1]-2.0) + x[1]*(x[1]-1.0))
		   end
	return nothing
end
xzero  = [5.]
xklagr = xzero
xkpen  = xzero
Lconv  = false
Pconv  = false
g      = [0.]
tol    = 1.0

@printf("it       mu           rho          La             P    \n")
@printf("------------------------------------------------------------\n")
#         0  1.000000e+01  1.000000e-01 -1.960000e+01  4.000000e-01
for k = 0:200
	rholagr = 0.1*1.1^k
	rhopen  = rholagr #0.1*1.6^k
	#readline()
	if ~Lconv && ~Pconv
		@printf("%2d %13.6e %13.6e %13.6e %13.6e\n",k,mu,rho,La(xklagr),P(xkpen))
	elseif ~Pconv
		@printf("%2d %13.6e %13.6e               %13.6e\n",k,mu,rho,P(xkpen))
	elseif ~Lconv
		@printf("%2d %13.6e %13.6e %13.6e\n",k,mu,rho,La(xklagr))
	else
		break
	end
	
	#," ",xklagr," ",xkpen)
	
	if ~Lconv
		xstart = xklagr
		rho = rholagr
		res = optimize(La,grad_La!,xstart,g_tol = tol,show_trace = false)
		#show(res)
		xklagr = res.minimum
		grad_La!(xklagr,g)
		if(norm(g) < 1.e-7) && (abs(h(xklagr)) < 1.e-7)
			Lconv = true
		end
		mu = mu + rho*h(xklagr)
	end
	if ~Pconv
		xstart = xkpen
		rho = rhopen
		res = optimize(P,grad_P!,xstart,g_tol = tol,show_trace = false)
		xkpen = res.minimum
		if(abs(h(xkpen)) < 1.e-7)
			Pconv = true
		end
	end
	
	tol = tol/2.0
	rho = rholagr
end
println(xklagr)
println(xkpen)

print("\n\nHit return key to proceed to next example")
readline()

################################
# min  x
# s.t. (x-1)(x-2)(x-3) = 0
#
# punto di KKT: x = 1, mu = -2
################################
@printf("\n\nMinimize x\n   s.t.  (x-1)(x-2)(x-3) = 0\n\n")

mu = 0.0
rho = 0

f(x) = x[1]
h(x) = (x[1]-1.0)*(x[1]-2.0)*(x[1]-3.0)

La(x) = f(x) + mu*h(x) + rho*(h(x)^2)
P(x)  = f(x) + rho*(h(x)^2)
function grad_P!(x,g)
	g[1] = 1.0 + 2.0*rho*(h(x))*((x[1]-1.0)*(x[1]-2.0) + (x[1]-3.0)*(x[1]-2.0) + (x[1]-3.0)*(x[1]-1.0))
	return nothing
end

function grad_La!(x,g)
	g[1] = begin
		   1.0 + mu*((x[1]-1.0)*(x[1]-2.0) + (x[1]-3.0)*(x[1]-2.0) + (x[1]-3.0)*(x[1]-1.0)) +
           2.0*rho*(h(x))*((x[1]-1.0)*(x[1]-2.0) + (x[1]-3.0)*(x[1]-2.0) + (x[1]-3.0)*(x[1]-1.0))
		   end
	return nothing
end
xzero  = [5.]
xklagr = xzero
xkpen  = xzero
Lconv  = false
Pconv  = false
g      = [0.]
tol    = 1.0

@printf("it       mu           rho          La             P    \n")
@printf("------------------------------------------------------------\n")
#         0  1.000000e+01  1.000000e-01 -1.960000e+01  4.000000e-01
for k = 0:200
	rholagr = 0.1*1.1^k
	rhopen  = rholagr #0.1*1.6^k
	#readline()
	if ~Lconv && ~Pconv
		@printf("%2d %13.6e %13.6e %13.6e %13.6e\n",k,mu,rho,La(xklagr),P(xkpen))
	elseif ~Pconv
		@printf("%2d %13.6e %13.6e               %13.6e\n",k,mu,rho,P(xkpen))
	elseif ~Lconv
		@printf("%2d %13.6e %13.6e %13.6e\n",k,mu,rho,La(xklagr))
	else
		break
	end
	
	#," ",xklagr," ",xkpen)
	
	if ~Lconv
		xstart = xklagr
		rho = rholagr
		res = optimize(La,grad_La!,xstart,g_tol = tol,show_trace = false)
		#show(res)
		xklagr = res.minimum
		grad_La!(xklagr,g)
		if(norm(g) < 1.e-7) && (abs(h(xklagr)) < 1.e-7)
			Lconv = true
		end
		mu = mu + rho*h(xklagr)
	end
	if ~Pconv
		xstart = xkpen
		rho = rhopen
		res = optimize(P,grad_P!,xstart,g_tol = tol,show_trace = false)
		xkpen = res.minimum
		if(abs(h(xkpen)) < 1.e-7)
			Pconv = true
		end
	end
	
	tol = max(tol/2.0,1.e-8)
	rho = rholagr
end
println(xklagr)
println(xkpen)

nothing

