using Optim
################################
# min  1/2*(x[1]^2 + (1/3)*x[2]^2)
# s.t. x[1] + x[2] = 1
#
# punti di KKT: 
#	x = (0.25,0.75) mu = -0.25
################################

mu = 0
rho = 0

f(x) = 1/2*(x[1]^2 + (1/3)*x[2]^2)
h(x) = x[1] + x[2] - 1.0

La(x) = f(x) + mu*h(x) + rho*(h(x))^2
P(x) = f(x) + rho*(h(x))^2
function grad_P!(x,g)
	g[1] = x[1] + 2.0*rho*h(x)
	g[2] = (1/3)*x[2] + 2.0*rho*h(x)
	return nothing
end

function grad_La!(x,g) 
	g[1] = x[1] + 2.0*rho*h(x) + mu
	g[2] = (1/3)*x[2] + 2.0*rho*h(x) + mu
	return nothing
end
xzero = [0. , 0.]
xklagr = xzero
xkpen  = xzero
Lconv  = false
Pconv  = false
g      = [0., 0.]

@printf("it       mu           rho       --------- xklagr ---------  ---------- xkpen ----------\n")
@printf("---------------------------------------------------------------------------------------\n")

for k = 0:300

	rho = 0.1*1.1^k
	#readline()
	if ~Lconv && ~Pconv
		@printf("%2d %13.6e %13.6e %13.6e %13.6e %13.6e %13.6e\n",k,mu,rho,xklagr[1],xklagr[2],xkpen[1],xkpen[2])
	elseif ~Pconv
		@printf("%2d %13.6e %13.6e                             %13.6e %13.6e\n",k,mu,rho,xkpen[1],xkpen[2])
	elseif ~Lconv
		@printf("%2d %13.6e %13.6e %13.6e %13.6e\n",k,mu,rho,xklagr[1],xklagr[2])
	else
		break
	end
	if ~Lconv	
		xstart = xklagr
		res = optimize(La,grad_La!,xstart,g_tol = 1e-8,show_trace = false)
		xklagr = res.minimum
		grad_La!(xklagr,g)
		if(norm(g) < 1.e-7) && (norm(h(xklagr)) < 1.e-7)
			Lconv = true
		end		
		mu = mu + rho*h(xklagr)
	end
	if ~Pconv
		xstart = xkpen
		res = optimize(P,grad_P!,xstart,g_tol = 1e-8,show_trace = false)
		xkpen = res.minimum
		if(norm(h(xkpen)) < 1.e-7)
			Pconv = true
		end		
	end
end
nothing

