using Optim
################################
# min  1/2*((x[2]+x[3])^2 + (x[1]+x[3])^2 + (x[1]+x[2])^2)
# s.t. x[1] + x[2] + 2x[3] = 2
#      x[1] - x[2] = 0
#
# punti di KKT: 
#	x = (0,0,1) mu = (-1 , 0)
################################

mu = [10.0, -5.]
rho = 0

f(x) = 1/2*((x[2]+x[3])^2 + (x[1]+x[3])^2 + (x[1]+x[2])^2)
h1(x) = x[1] + x[2] + 2*x[3] - 2.0
h2(x) = x[1] - x[2]
h(x) = [h1(x) h2(x)]

La(x) = f(x) + mu[1]*h1(x) + mu[2]*h2(x) + rho*(h1(x)^2+h2(x)^2)
P(x)  = f(x) + rho*(h1(x)^2+h2(x)^2)
function grad_P!(x,g)
	g[1] = x[1] + x[3] + x[1] + x[2] + 2.0*rho*(h1(x) + h2(x))
	g[2] = x[2] + x[3] + x[1] + x[2] + 2.0*rho*(h1(x) - h2(x))
	g[3] = x[2] + x[3] + x[1] + x[3] + 4.0*rho*(h1(x))
	return nothing
end

function grad_La!(x,g) 
	g[1] = x[1] + x[3] + x[1] + x[2] + 2.0*rho*(h1(x) + h2(x)) + mu[1] + mu[2]
	g[2] = x[2] + x[3] + x[1] + x[2] + 2.0*rho*(h1(x) - h2(x)) + mu[1] - mu[2]
	g[3] = x[2] + x[3] + x[1] + x[3] + 4.0*rho*(h1(x))     + 2.0*mu[1]
	return nothing
end
xzero  = [0. , 0. , 0.]
xklagr = xzero
xkpen  = xzero
g      = copy(xzero)
Lconv  = false
Pconv  = false
delta  = 20
tol    = 1.0

@printf("it  ------------mu------------       rho          La             P    \n")
@printf("--------------------------------------------------------------------------\n")
#         0  1.000000e+01 -5.000000e+00  1.000000e-01 -1.960000e+01  4.000000e-01
for k = 0:300
	rho = 0.1*1.1^k

	if ~Lconv && ~Pconv
		@printf("%2d %13.6e %13.6e %13.6e %13.6e %13.6e\n",k,mu[1],mu[2],rho,La(xklagr),P(xkpen))
	elseif ~Pconv
		@printf("%2d %13.6e %13.6e %13.6e               %13.6e\n",k,mu[1],mu[2],rho,P(xkpen))
	elseif ~Lconv
		@printf("%2d %13.6e %13.6e %13.6e %13.6e\n",k,mu[1],mu[2],rho,La(xklagr))
	else
		break
	end
	if ~Lconv	
		xstart = xklagr
		res = optimize(La,grad_La!,xstart,g_tol = tol,show_trace = false)
		xklagr = res.minimum
		grad_La!(xklagr,g)
		if norm(g) < 1.e-6 && norm(h(xklagr)) < 1.e-6
			Lconv = true
		end
		alpha = 2.0*rho*(1-rho/(delta + 2.0*rho))

		mu[1] = mu[1] + alpha*h1(xklagr)
		mu[2] = mu[2] + alpha*h2(xklagr)
	end
	if ~Pconv
		xstart = xkpen
		res = optimize(P,grad_P!,xstart,g_tol = tol,show_trace = false)
		xkpen = res.minimum
		if norm(h(xkpen)) < 1.e-6
			Pconv = true
		end
	end
	tol = max(tol/2,1.e-8)
end
nothing

